#include "global.h"
#include "dialogimpl.h"
#include "socket.h"
//
DialogImpl::DialogImpl( QWidget * parent, Qt::WFlags f) 
	: QDialog(parent, f)
{
	socket_init(this);
	setupUi(this);
	
	lineEdit_Server->setText("www.baidu.com");
	lineEdit_Port->setText("80");
	radioButton_Get->setChecked(TRUE);
	radioButton_HTTP11->setChecked(TRUE);
	lineEdit_Request->setText("/");
	lineEdit_Referer->setText("http://www.baidu.com");
	lineEdit_Host->setText("www.baidu.com");
	lineEdit_Accept->setText("*/*");
	lineEdit_AcceptEncoding->setText("gzip, deflate");
	lineEdit_AcceptLanguage->setText("zh-cn");
	lineEdit_CacheControl->setText("");
	lineEdit_UserAgent->setText("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; Maxthon; .NET CLR 2.0.50727)");
	lineEdit_Connection->setText("Keep-Alive");
	lineEdit_Cookie->setText("");
	lineEdit_PostContent->setText("");
	
	connect( ButtonBegin, SIGNAL(clicked()), this, SLOT(Begin()) );
}
//

void DialogImpl::Begin( void )
{
	int ret, socket_fd;
	char const *p;
	char *buffer;

	// check the data first.
	if (lineEdit_Server->text().isEmpty()) {
		ShowMessage("Please input the server(name or IP).");
		return;
	}
	if (lineEdit_Port->text().isEmpty()) {
		ShowMessage("Please input the server port(the default web port is 80).");
		return;
	}
	if (lineEdit_Request->text().isEmpty()) {
		ShowMessage("Please input the request.");
		return;
	}
	if (radioButton_Post->isChecked() && lineEdit_PostContent->text().isEmpty()) {
		ShowMessage("Please input the post content, otherwise please select 'GET'.");
		return;
	}
	
	// connect to the server.
	socket_fd = socket_connect(lineEdit_Server->text().toUtf8().data(), lineEdit_Port->text().toInt());
	if (socket_fd <= 0) {
		return;
	}

	buffer = (char *)calloc(1, MAX_WEBPAGE_SIZE);
	if (buffer == NULL) {
		ShowMessage("Not enough memory(%s)!", MAX_WEBPAGE_SIZE);
		return;
	}
	
	// prepare to send the data.
	ShowMessage("Prepare to send the data to server.");
	if (radioButton_HTTP11->isChecked()) {
		p = "HTTP/1.1";
	} else {
		p = "HTTP/1.0";
	}
	if (radioButton_Post->isChecked()) {
		// POST
		sprintf(buffer,  "POST %s %s\n", lineEdit_Request->text().toUtf8().data(), p);
	} else {
		// GET
		sprintf(buffer,  "GET %s %s\n", lineEdit_Request->text().toUtf8().data(), p);
	}
	ret = socket_send(socket_fd, strlen(buffer), buffer);
	if (ret < 0) {
		ShowMessage("Send data fail(%s).", buffer);
		goto finish;
	}
	ShowMessage(buffer);
	// referer
	if (!lineEdit_Referer->text().isEmpty()) {
		sprintf(buffer,  "Referer: %s\n", lineEdit_Referer->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// host
	if (!lineEdit_Host->text().isEmpty()) {
		sprintf(buffer,  "Host: %s\n", lineEdit_Host->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// accept
	if (!lineEdit_Accept->text().isEmpty()) {
		sprintf(buffer,  "Accept: %s\n", lineEdit_Accept->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Accept-Encoding
	if (!lineEdit_AcceptEncoding->text().isEmpty()) {
		sprintf(buffer,  "Accept-Encoding: %s\n", lineEdit_AcceptEncoding->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Accept-Language
	if (!lineEdit_AcceptLanguage->text().isEmpty()) {
		sprintf(buffer,  "Accept-Language: %s\n", lineEdit_AcceptLanguage->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Cache-Control
	if (!lineEdit_CacheControl->text().isEmpty()) {
		sprintf(buffer,  "Cache-Control: %s\n", lineEdit_CacheControl->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Connection
	if (!lineEdit_Connection->text().isEmpty()) {
		sprintf(buffer,  "Connection: %s\n", lineEdit_Connection->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Cookie
	if (!lineEdit_Cookie->text().isEmpty()) {
		sprintf(buffer,  "Cookie: %s\n", lineEdit_Cookie->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Name1
	if (!lineEdit_Name1->text().isEmpty()) {
		sprintf(buffer,  "%s: %s\n", lineEdit_Name1->text().toUtf8().data(), lineEdit_Value1->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Name2
	if (!lineEdit_Name2->text().isEmpty()) {
		sprintf(buffer,  "%s: %s\n", lineEdit_Name2->text().toUtf8().data(), lineEdit_Value2->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// Name3
	if (!lineEdit_Name3->text().isEmpty()) {
		sprintf(buffer,  "%s: %s\n", lineEdit_Name3->text().toUtf8().data(), lineEdit_Value3->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// POST Content Length
	if (radioButton_Post->isChecked()) {
		sprintf(buffer,  "Content-Length: %d\n", lineEdit_PostContent->text().size());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	// the end flag.
	sprintf(buffer,  "\n");
	ret = socket_send(socket_fd, strlen(buffer), buffer);
	if (ret < 0) {
		ShowMessage("Send data fail(%s).", buffer);
		goto finish;
	}
	ShowMessage(buffer);
	// POST Content
	if (radioButton_Post->isChecked()) {
		sprintf(buffer,  "%s", lineEdit_PostContent->text().toUtf8().data());
		ret = socket_send(socket_fd, strlen(buffer), buffer);
		if (ret < 0) {
			ShowMessage("Send data fail(%s).", buffer);
			goto finish;
		}
		ShowMessage(buffer);
	}
	ShowMessage("Send the data finished.");
	ShowMessage("Prepare to receive the data.");
	ret = socket_receive(socket_fd, MAX_WEBPAGE_SIZE, buffer);
	if (ret <= 0) {
		ShowMessage("Receive the data fail.");
	} else {
		ShowMessage("Receive the data:");
		ShowMessage(buffer);
	}
	
finish:
	free(buffer);
	// close socket.
	ret = socket_close(socket_fd);
	if (ret == 0) {
		ShowMessage("Close the connection successful.");
	} else {
		ShowMessage("Close the connection fail.");
	}
}

void DialogImpl::ShowMessage(const char *fmt, ...)
{
	va_list	args;
	char buf[BUFFERLENGTH];

	va_start(args, fmt);
	vsprintf(buf, fmt, args);
	va_end(args);
	listWidget->addItem(buf);
}
